<?php

namespace Drupal\commerce_eurobank_redirect\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\commerce_order\Entity\OrderInterface;

/**
 * Endpoints for the routes defined.
 */
class CallbackController extends ControllerBase {
  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   *
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LoggerChannelFactoryInterface $logger_factory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->logger = $logger_factory->get('commerce_eurobank_redirect');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.factory')
    );
  }

  /**
   * Callback action.
   *
   * Listen for callbacks from QuickPay and creates any payment specified.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function callback(Request $request) {
    $message = $this->processCallback($request);
    return [
      '#type' => 'markup',
      '#markup' => $message,
    ];
  }

  /**
   * Process the callback from eurobank.
   */
  public function processCallback(Request $request) {
    $action = $request->get("action");
	
    $resultCode = $request->get('status');
    if ($resultCode !== "CAPTURED" && $resultCode !== "AUTHORIZED" ) {
      $this->logResponse($request, "failure");
      return $this->getDisplayMessage($request);
    }

    $digest = $request->get('digest');
    $order_id = $request->get('orderid');
    $order = $this->entityTypeManager->getStorage('commerce_order')->load($order_id);

    $hash = $this->calculateHash($request, $order_id);
    if ($hash !== $digest) {
      $this->createPayment($order, $request, "Unvalidated");
      $this->logResponse($request, "failure");
      return t('Sorry we were not able to validate your payment');
    }
    $this->createPayment($order, $request);
    $this->logResponse($request, "success");
    return $this->getDisplayMessage($request);
  }

  /**
   * Log response data to database log and order object.
   */
  public function logResponse($request, $status = "success") {
    $order_id = $request->get('orderid');
    $data = [
      "txId" => $request->get('txId'),
      "orderid" => $order_id,
      "status" => $request->get('status'),
    ];

    // log for immediate action
    if($status === "success"){
      $this->logger->info("Payment " . $status . ": " . '<pre><code>' . print_r($data, TRUE) . '</code></pre>');
    }else if($status === "failure"){
      $this->logger->alert("Payment " . $status . ": " . '<pre><code>' . print_r($data, TRUE) . '</code></pre>');
    }
	
	// log in the order data field for future reference
    $order = $this->entityTypeManager->getStorage('commerce_order')->load($order_id);
    $EurobankGatewayData = $order->getData("EurobankGatewayData");
    $EurobankGatewayData = array_merge($EurobankGatewayData, $data);
    $order->setData("EurobankGatewayData", $EurobankGatewayData);
    $order->save();
	
  }

  /**
   * Create Payment.
   */
   
  public function createPayment(OrderInterface $order, Request $request, $state = "completed") {
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
	$gateway_data = $order->getData("EurobankGatewayData");
    $payment = $payment_storage->create([
      'state' => 'completed',
      'amount' => $order->getBalance(),
	  'payment_gateway' => $gateway_data["payment_gateway"],
      'order_id' => $order->id(),
      'remote_state' => $request->get('status'),
	  'message' => $request->get('message'),
	  'payMethod' => $request->get('payMethod'),
      'remote_id' => $request->get('txId'),
	  'paymentRef' => $request->get('paymentRef'),
    ]);
    // if ($state == "completed") {
      // $payment->setAuthorizedTime(REQUEST_TIME);
      $payment->setCompletedTime(REQUEST_TIME);
    // }
    $payment->save();
	drupal_set_message($this->t('Your payment was successful with order_id : @order_id', [
		'@order_id' => $order->id(),
	]));
  }
  
   /**
   * Calculates hash key by concatenation of values, then uses sha256 algorithm.
   *
   * @param $request
   *
   * @param $order_id
   *
   * @return string
   */
  public function calculateHash($request, $order_id) {
	$order = $this->entityTypeManager->getStorage('commerce_order')->load($order_id);
	$gateway_data = $order->getData("EurobankGatewayData");
	$post_data_array = array();	  
	$post_data_array[0] = $request->get('version');
	$post_data_array[1] = $request->get('mid');
	$post_data_array[2] = $order_id;
	$post_data_array[3] = $request->get('status');
	$post_data_array[4] = $request->get('orderAmount');
	$post_data_array[5] = $request->get('currency');
	$post_data_array[6] = $request->get('paymentTotal');
	$post_data_array[7] = $request->get('message');
	$post_data_array[8] = $request->get('riskScore');
	$post_data_array[9] = $request->get('payMethod');
	$post_data_array[10] = $request->get('txId');
	$post_data_array[11] = $request->get('Sequence');
	$post_data_array[12] = $request->get('SeqTxId');
	$post_data_array[13] = $request->get('paymentRef');	
	$post_data_array[14] = $gateway_data["shared_secret"];
	$post_data = implode("", $post_data_array);
	return base64_encode(hash("sha256",$post_data,true));
  }	
	
  /**
   * Get message .
   *
   * @param $request
   *
   * @return string
   */
  public function getDisplayMessage($request) {
      $messages = [
        'AUTHORIZED,' => t('Payment was successful (accept order)'),
        'CAPTURED' => t('Payment was successful (accept order)'),
        'CANCELED' => t('Payment failed, user canceled the process (deny order)'),
        'REFUSED' => t('Payment failed, payment was denied for card or by bank (deny order)'),
        'ERROR' => t('Non recoverable system or other error occurred during payment process (deny order)'),
      ];
      $message = $messages[$request->get('status')];
    return $message;
  }

}
